# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Required, TypedDict

from .price_param import PriceParam
from .tax_category import TaxCategory
from .license_key_duration_param import LicenseKeyDurationParam

__all__ = ["ProductCreateParams"]


class ProductCreateParams(TypedDict, total=False):
    price: Required[PriceParam]

    tax_category: Required[TaxCategory]
    """
    Represents the different categories of taxation applicable to various products
    and services.
    """

    addons: Optional[List[str]]
    """Addons available for subscription product"""

    brand_id: Optional[str]
    """Brand id for the product, if not provided will default to primary brand"""

    description: Optional[str]
    """Optional description of the product"""

    license_key_activation_message: Optional[str]
    """Optional message displayed during license key activation"""

    license_key_activations_limit: Optional[int]
    """The number of times the license key can be activated. Must be 0 or greater"""

    license_key_duration: Optional[LicenseKeyDurationParam]

    license_key_enabled: Optional[bool]
    """
    When true, generates and sends a license key to your customer. Defaults to false
    """

    name: Optional[str]
    """Optional name of the product"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .price import Price as Price
from .refund import Refund as Refund
from .dispute import Dispute as Dispute
from .payment import Payment as Payment
from .product import Product as Product
from .currency import Currency as Currency
from .customer import Customer as Customer
from .discount import Discount as Discount
from .license_key import LicenseKey as LicenseKey
from .price_param import PriceParam as PriceParam
from .country_code import CountryCode as CountryCode
from .subscription import Subscription as Subscription
from .tax_category import TaxCategory as TaxCategory
from .discount_type import DiscountType as DiscountType
from .dispute_stage import DisputeStage as DisputeStage
from .intent_status import IntentStatus as IntentStatus
from .refund_status import RefundStatus as RefundStatus
from .time_interval import TimeInterval as TimeInterval
from .webhook_event import WebhookEvent as WebhookEvent
from .addon_response import AddonResponse as AddonResponse
from .dispute_status import DisputeStatus as DisputeStatus
from .billing_address import BillingAddress as BillingAddress
from .addon_list_params import AddonListParams as AddonListParams
from .license_key_status import LicenseKeyStatus as LicenseKeyStatus
from .payout_list_params import PayoutListParams as PayoutListParams
from .refund_list_params import RefundListParams as RefundListParams
from .addon_create_params import AddonCreateParams as AddonCreateParams
from .addon_update_params import AddonUpdateParams as AddonUpdateParams
from .brand_create_params import BrandCreateParams as BrandCreateParams
from .brand_list_response import BrandListResponse as BrandListResponse
from .brand_update_params import BrandUpdateParams as BrandUpdateParams
from .dispute_list_params import DisputeListParams as DisputeListParams
from .payment_list_params import PaymentListParams as PaymentListParams
from .product_list_params import ProductListParams as ProductListParams
from .subscription_status import SubscriptionStatus as SubscriptionStatus
from .customer_list_params import CustomerListParams as CustomerListParams
from .discount_list_params import DiscountListParams as DiscountListParams
from .license_key_duration import LicenseKeyDuration as LicenseKeyDuration
from .license_key_instance import LicenseKeyInstance as LicenseKeyInstance
from .payout_list_response import PayoutListResponse as PayoutListResponse
from .refund_create_params import RefundCreateParams as RefundCreateParams
from .billing_address_param import BillingAddressParam as BillingAddressParam
from .brand_create_response import BrandCreateResponse as BrandCreateResponse
from .brand_update_response import BrandUpdateResponse as BrandUpdateResponse
from .dispute_list_response import DisputeListResponse as DisputeListResponse
from .payment_create_params import PaymentCreateParams as PaymentCreateParams
from .payment_list_response import PaymentListResponse as PaymentListResponse
from .product_create_params import ProductCreateParams as ProductCreateParams
from .product_list_response import ProductListResponse as ProductListResponse
from .product_update_params import ProductUpdateParams as ProductUpdateParams
from .customer_create_params import CustomerCreateParams as CustomerCreateParams
from .customer_request_param import CustomerRequestParam as CustomerRequestParam
from .customer_update_params import CustomerUpdateParams as CustomerUpdateParams
from .discount_create_params import DiscountCreateParams as DiscountCreateParams
from .discount_update_params import DiscountUpdateParams as DiscountUpdateParams
from .brand_retrieve_response import BrandRetrieveResponse as BrandRetrieveResponse
from .customer_portal_session import CustomerPortalSession as CustomerPortalSession
from .license_activate_params import LicenseActivateParams as LicenseActivateParams
from .license_key_list_params import LicenseKeyListParams as LicenseKeyListParams
from .license_validate_params import LicenseValidateParams as LicenseValidateParams
from .payment_create_response import PaymentCreateResponse as PaymentCreateResponse
from .addon_cart_response_item import AddonCartResponseItem as AddonCartResponseItem
from .customer_limited_details import CustomerLimitedDetails as CustomerLimitedDetails
from .subscription_list_params import SubscriptionListParams as SubscriptionListParams
from .create_new_customer_param import CreateNewCustomerParam as CreateNewCustomerParam
from .dispute_retrieve_response import DisputeRetrieveResponse as DisputeRetrieveResponse
from .license_deactivate_params import LicenseDeactivateParams as LicenseDeactivateParams
from .license_key_update_params import LicenseKeyUpdateParams as LicenseKeyUpdateParams
from .license_validate_response import LicenseValidateResponse as LicenseValidateResponse
from .webhook_event_list_params import WebhookEventListParams as WebhookEventListParams
from .license_key_duration_param import LicenseKeyDurationParam as LicenseKeyDurationParam
from .one_time_product_cart_item import OneTimeProductCartItem as OneTimeProductCartItem
from .subscription_charge_params import SubscriptionChargeParams as SubscriptionChargeParams
from .subscription_create_params import SubscriptionCreateParams as SubscriptionCreateParams
from .subscription_list_response import SubscriptionListResponse as SubscriptionListResponse
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams
from .addon_update_images_response import AddonUpdateImagesResponse as AddonUpdateImagesResponse
from .brand_update_images_response import BrandUpdateImagesResponse as BrandUpdateImagesResponse
from .subscription_charge_response import SubscriptionChargeResponse as SubscriptionChargeResponse
from .subscription_create_response import SubscriptionCreateResponse as SubscriptionCreateResponse
from .attach_existing_customer_param import AttachExistingCustomerParam as AttachExistingCustomerParam
from .subscription_change_plan_params import SubscriptionChangePlanParams as SubscriptionChangePlanParams
from .license_key_instance_list_params import LicenseKeyInstanceListParams as LicenseKeyInstanceListParams
from .one_time_product_cart_item_param import OneTimeProductCartItemParam as OneTimeProductCartItemParam
from .license_key_instance_update_params import LicenseKeyInstanceUpdateParams as LicenseKeyInstanceUpdateParams
from .payment_retrieve_line_items_response import PaymentRetrieveLineItemsResponse as PaymentRetrieveLineItemsResponse
from .misc_list_supported_countries_response import (
    MiscListSupportedCountriesResponse as MiscListSupportedCountriesResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

import httpx

from .images import (
    ImagesResource,
    AsyncImagesResource,
    ImagesResourceWithRawResponse,
    AsyncImagesResourceWithRawResponse,
    ImagesResourceWithStreamingResponse,
    AsyncImagesResourceWithStreamingResponse,
)
from ...types import (
    TaxCategory,
    product_list_params,
    product_create_params,
    product_update_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.product import Product
from ...types.price_param import PriceParam
from ...types.tax_category import TaxCategory
from ...types.product_list_response import ProductListResponse
from ...types.license_key_duration_param import LicenseKeyDurationParam

__all__ = ["ProductsResource", "AsyncProductsResource"]


class ProductsResource(SyncAPIResource):
    @cached_property
    def images(self) -> ImagesResource:
        return ImagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return ProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return ProductsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        price: PriceParam,
        tax_category: TaxCategory,
        addons: Optional[List[str]] | NotGiven = NOT_GIVEN,
        brand_id: Optional[str] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activation_message: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activations_limit: Optional[int] | NotGiven = NOT_GIVEN,
        license_key_duration: Optional[LicenseKeyDurationParam] | NotGiven = NOT_GIVEN,
        license_key_enabled: Optional[bool] | NotGiven = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Product:
        """
        Args:
          tax_category: Represents the different categories of taxation applicable to various products
              and services.

          addons: Addons available for subscription product

          brand_id: Brand id for the product, if not provided will default to primary brand

          description: Optional description of the product

          license_key_activation_message: Optional message displayed during license key activation

          license_key_activations_limit: The number of times the license key can be activated. Must be 0 or greater

          license_key_enabled: When true, generates and sends a license key to your customer. Defaults to false

          name: Optional name of the product

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/products",
            body=maybe_transform(
                {
                    "price": price,
                    "tax_category": tax_category,
                    "addons": addons,
                    "brand_id": brand_id,
                    "description": description,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "name": name,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Product:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def update(
        self,
        id: str,
        *,
        addons: Optional[List[str]] | NotGiven = NOT_GIVEN,
        brand_id: Optional[str] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        image_id: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activation_message: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activations_limit: Optional[int] | NotGiven = NOT_GIVEN,
        license_key_duration: Optional[LicenseKeyDurationParam] | NotGiven = NOT_GIVEN,
        license_key_enabled: Optional[bool] | NotGiven = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        price: Optional[PriceParam] | NotGiven = NOT_GIVEN,
        tax_category: Optional[TaxCategory] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          addons: Available Addons for subscription products

          description: Description of the product, optional and must be at most 1000 characters.

          image_id: Product image id after its uploaded to S3

          license_key_activation_message: Message sent to the customer upon license key activation.

              Only applicable if `license_key_enabled` is `true`. This message contains
              instructions for activating the license key.

          license_key_activations_limit: Limit for the number of activations for the license key.

              Only applicable if `license_key_enabled` is `true`. Represents the maximum
              number of times the license key can be activated.

          license_key_enabled: Whether the product requires a license key.

              If `true`, additional fields related to license key (duration, activations
              limit, activation message) become applicable.

          name: Name of the product, optional and must be at most 100 characters.

          tax_category: Represents the different categories of taxation applicable to various products
              and services.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/products/{id}",
            body=maybe_transform(
                {
                    "addons": addons,
                    "brand_id": brand_id,
                    "description": description,
                    "image_id": image_id,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "name": name,
                    "price": price,
                    "tax_category": tax_category,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        archived: bool | NotGiven = NOT_GIVEN,
        brand_id: Optional[str] | NotGiven = NOT_GIVEN,
        page_number: Optional[int] | NotGiven = NOT_GIVEN,
        page_size: Optional[int] | NotGiven = NOT_GIVEN,
        recurring: Optional[bool] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncDefaultPageNumberPagination[ProductListResponse]:
        """
        Args:
          archived: List archived products

          brand_id: filter by Brand id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          recurring:
              Filter products by pricing type:

              - `true`: Show only recurring pricing products (e.g. subscriptions)
              - `false`: Show only one-time price products
              - `null` or absent: Show both types of products

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/products",
            page=SyncDefaultPageNumberPagination[ProductListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "brand_id": brand_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "recurring": recurring,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/products/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncProductsResource(AsyncAPIResource):
    @cached_property
    def images(self) -> AsyncImagesResource:
        return AsyncImagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncProductsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        price: PriceParam,
        tax_category: TaxCategory,
        addons: Optional[List[str]] | NotGiven = NOT_GIVEN,
        brand_id: Optional[str] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activation_message: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activations_limit: Optional[int] | NotGiven = NOT_GIVEN,
        license_key_duration: Optional[LicenseKeyDurationParam] | NotGiven = NOT_GIVEN,
        license_key_enabled: Optional[bool] | NotGiven = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Product:
        """
        Args:
          tax_category: Represents the different categories of taxation applicable to various products
              and services.

          addons: Addons available for subscription product

          brand_id: Brand id for the product, if not provided will default to primary brand

          description: Optional description of the product

          license_key_activation_message: Optional message displayed during license key activation

          license_key_activations_limit: The number of times the license key can be activated. Must be 0 or greater

          license_key_enabled: When true, generates and sends a license key to your customer. Defaults to false

          name: Optional name of the product

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/products",
            body=await async_maybe_transform(
                {
                    "price": price,
                    "tax_category": tax_category,
                    "addons": addons,
                    "brand_id": brand_id,
                    "description": description,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "name": name,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Product:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def update(
        self,
        id: str,
        *,
        addons: Optional[List[str]] | NotGiven = NOT_GIVEN,
        brand_id: Optional[str] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        image_id: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activation_message: Optional[str] | NotGiven = NOT_GIVEN,
        license_key_activations_limit: Optional[int] | NotGiven = NOT_GIVEN,
        license_key_duration: Optional[LicenseKeyDurationParam] | NotGiven = NOT_GIVEN,
        license_key_enabled: Optional[bool] | NotGiven = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        price: Optional[PriceParam] | NotGiven = NOT_GIVEN,
        tax_category: Optional[TaxCategory] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          addons: Available Addons for subscription products

          description: Description of the product, optional and must be at most 1000 characters.

          image_id: Product image id after its uploaded to S3

          license_key_activation_message: Message sent to the customer upon license key activation.

              Only applicable if `license_key_enabled` is `true`. This message contains
              instructions for activating the license key.

          license_key_activations_limit: Limit for the number of activations for the license key.

              Only applicable if `license_key_enabled` is `true`. Represents the maximum
              number of times the license key can be activated.

          license_key_enabled: Whether the product requires a license key.

              If `true`, additional fields related to license key (duration, activations
              limit, activation message) become applicable.

          name: Name of the product, optional and must be at most 100 characters.

          tax_category: Represents the different categories of taxation applicable to various products
              and services.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/products/{id}",
            body=await async_maybe_transform(
                {
                    "addons": addons,
                    "brand_id": brand_id,
                    "description": description,
                    "image_id": image_id,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "name": name,
                    "price": price,
                    "tax_category": tax_category,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        archived: bool | NotGiven = NOT_GIVEN,
        brand_id: Optional[str] | NotGiven = NOT_GIVEN,
        page_number: Optional[int] | NotGiven = NOT_GIVEN,
        page_size: Optional[int] | NotGiven = NOT_GIVEN,
        recurring: Optional[bool] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ProductListResponse, AsyncDefaultPageNumberPagination[ProductListResponse]]:
        """
        Args:
          archived: List archived products

          brand_id: filter by Brand id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          recurring:
              Filter products by pricing type:

              - `true`: Show only recurring pricing products (e.g. subscriptions)
              - `false`: Show only one-time price products
              - `null` or absent: Show both types of products

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/products",
            page=AsyncDefaultPageNumberPagination[ProductListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "brand_id": brand_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "recurring": recurring,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/products/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ProductsResourceWithRawResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = to_raw_response_wrapper(
            products.update,
        )
        self.list = to_raw_response_wrapper(
            products.list,
        )
        self.delete = to_raw_response_wrapper(
            products.delete,
        )
        self.unarchive = to_raw_response_wrapper(
            products.unarchive,
        )

    @cached_property
    def images(self) -> ImagesResourceWithRawResponse:
        return ImagesResourceWithRawResponse(self._products.images)


class AsyncProductsResourceWithRawResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            products.update,
        )
        self.list = async_to_raw_response_wrapper(
            products.list,
        )
        self.delete = async_to_raw_response_wrapper(
            products.delete,
        )
        self.unarchive = async_to_raw_response_wrapper(
            products.unarchive,
        )

    @cached_property
    def images(self) -> AsyncImagesResourceWithRawResponse:
        return AsyncImagesResourceWithRawResponse(self._products.images)


class ProductsResourceWithStreamingResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            products.update,
        )
        self.list = to_streamed_response_wrapper(
            products.list,
        )
        self.delete = to_streamed_response_wrapper(
            products.delete,
        )
        self.unarchive = to_streamed_response_wrapper(
            products.unarchive,
        )

    @cached_property
    def images(self) -> ImagesResourceWithStreamingResponse:
        return ImagesResourceWithStreamingResponse(self._products.images)


class AsyncProductsResourceWithStreamingResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            products.update,
        )
        self.list = async_to_streamed_response_wrapper(
            products.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            products.delete,
        )
        self.unarchive = async_to_streamed_response_wrapper(
            products.unarchive,
        )

    @cached_property
    def images(self) -> AsyncImagesResourceWithStreamingResponse:
        return AsyncImagesResourceWithStreamingResponse(self._products.images)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .country_code import CountryCode

__all__ = ["BillingAddressParam"]


class BillingAddressParam(TypedDict, total=False):
    city: Required[str]
    """City name"""

    country: Required[CountryCode]
    """ISO country code alpha2 variant"""

    state: Required[str]
    """State or province name"""

    street: Required[str]
    """Street address including house number and unit/apartment if applicable"""

    zipcode: Required[str]
    """Postal code or ZIP code"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    BrandListResponse,
    BrandCreateResponse,
    BrandUpdateResponse,
    BrandRetrieveResponse,
    BrandUpdateImagesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBrands:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: DodoPayments) -> None:
        brand = client.brands.create()
        assert_matches_type(BrandCreateResponse, brand, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: DodoPayments) -> None:
        brand = client.brands.create(
            description="description",
            name="name",
            statement_descriptor="statement_descriptor",
            support_email="support_email",
            url="url",
        )
        assert_matches_type(BrandCreateResponse, brand, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: DodoPayments) -> None:
        response = client.brands.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandCreateResponse, brand, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: DodoPayments) -> None:
        with client.brands.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandCreateResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        brand = client.brands.retrieve(
            "id",
        )
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.brands.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.brands.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.brands.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: DodoPayments) -> None:
        brand = client.brands.update(
            id="id",
        )
        assert_matches_type(BrandUpdateResponse, brand, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: DodoPayments) -> None:
        brand = client.brands.update(
            id="id",
            image_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            statement_descriptor="statement_descriptor",
            support_email="support_email",
        )
        assert_matches_type(BrandUpdateResponse, brand, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: DodoPayments) -> None:
        response = client.brands.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandUpdateResponse, brand, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: DodoPayments) -> None:
        with client.brands.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandUpdateResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.brands.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: DodoPayments) -> None:
        brand = client.brands.list()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: DodoPayments) -> None:
        response = client.brands.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: DodoPayments) -> None:
        with client.brands.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandListResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_update_images(self, client: DodoPayments) -> None:
        brand = client.brands.update_images(
            "id",
        )
        assert_matches_type(BrandUpdateImagesResponse, brand, path=["response"])

    @parametrize
    def test_raw_response_update_images(self, client: DodoPayments) -> None:
        response = client.brands.with_raw_response.update_images(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandUpdateImagesResponse, brand, path=["response"])

    @parametrize
    def test_streaming_response_update_images(self, client: DodoPayments) -> None:
        with client.brands.with_streaming_response.update_images(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandUpdateImagesResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_images(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.brands.with_raw_response.update_images(
                "",
            )


class TestAsyncBrands:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.create()
        assert_matches_type(BrandCreateResponse, brand, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.create(
            description="description",
            name="name",
            statement_descriptor="statement_descriptor",
            support_email="support_email",
            url="url",
        )
        assert_matches_type(BrandCreateResponse, brand, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.brands.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandCreateResponse, brand, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.brands.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandCreateResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.retrieve(
            "id",
        )
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.brands.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.brands.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.brands.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.update(
            id="id",
        )
        assert_matches_type(BrandUpdateResponse, brand, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.update(
            id="id",
            image_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            statement_descriptor="statement_descriptor",
            support_email="support_email",
        )
        assert_matches_type(BrandUpdateResponse, brand, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.brands.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandUpdateResponse, brand, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.brands.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandUpdateResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.brands.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.list()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.brands.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.brands.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandListResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_update_images(self, async_client: AsyncDodoPayments) -> None:
        brand = await async_client.brands.update_images(
            "id",
        )
        assert_matches_type(BrandUpdateImagesResponse, brand, path=["response"])

    @parametrize
    async def test_raw_response_update_images(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.brands.with_raw_response.update_images(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandUpdateImagesResponse, brand, path=["response"])

    @parametrize
    async def test_streaming_response_update_images(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.brands.with_streaming_response.update_images(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandUpdateImagesResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_images(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.brands.with_raw_response.update_images(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Dict, Union, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    misc,
    addons,
    brands,
    payouts,
    refunds,
    disputes,
    licenses,
    payments,
    discounts,
    license_keys,
    subscriptions,
    webhook_events,
    license_key_instances,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, DodoPaymentsError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.invoices import invoices
from .resources.products import products
from .resources.customers import customers

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "DodoPayments",
    "AsyncDodoPayments",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "live_mode": "https://live.dodopayments.com",
    "test_mode": "https://test.dodopayments.com",
}


class DodoPayments(SyncAPIClient):
    payments: payments.PaymentsResource
    subscriptions: subscriptions.SubscriptionsResource
    invoices: invoices.InvoicesResource
    licenses: licenses.LicensesResource
    license_keys: license_keys.LicenseKeysResource
    license_key_instances: license_key_instances.LicenseKeyInstancesResource
    customers: customers.CustomersResource
    refunds: refunds.RefundsResource
    disputes: disputes.DisputesResource
    payouts: payouts.PayoutsResource
    webhook_events: webhook_events.WebhookEventsResource
    products: products.ProductsResource
    misc: misc.MiscResource
    discounts: discounts.DiscountsResource
    addons: addons.AddonsResource
    brands: brands.BrandsResource
    with_raw_response: DodoPaymentsWithRawResponse
    with_streaming_response: DodoPaymentsWithStreamedResponse

    # client options
    bearer_token: str

    _environment: Literal["live_mode", "test_mode"] | NotGiven

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        environment: Literal["live_mode", "test_mode"] | NotGiven = NOT_GIVEN,
        base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous DodoPayments client instance.

        This automatically infers the `bearer_token` argument from the `DODO_PAYMENTS_API_KEY` environment variable if it is not provided.
        """
        if bearer_token is None:
            bearer_token = os.environ.get("DODO_PAYMENTS_API_KEY")
        if bearer_token is None:
            raise DodoPaymentsError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the DODO_PAYMENTS_API_KEY environment variable"
            )
        self.bearer_token = bearer_token

        self._environment = environment

        base_url_env = os.environ.get("DODO_PAYMENTS_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `DODO_PAYMENTS_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "live_mode"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.payments = payments.PaymentsResource(self)
        self.subscriptions = subscriptions.SubscriptionsResource(self)
        self.invoices = invoices.InvoicesResource(self)
        self.licenses = licenses.LicensesResource(self)
        self.license_keys = license_keys.LicenseKeysResource(self)
        self.license_key_instances = license_key_instances.LicenseKeyInstancesResource(self)
        self.customers = customers.CustomersResource(self)
        self.refunds = refunds.RefundsResource(self)
        self.disputes = disputes.DisputesResource(self)
        self.payouts = payouts.PayoutsResource(self)
        self.webhook_events = webhook_events.WebhookEventsResource(self)
        self.products = products.ProductsResource(self)
        self.misc = misc.MiscResource(self)
        self.discounts = discounts.DiscountsResource(self)
        self.addons = addons.AddonsResource(self)
        self.brands = brands.BrandsResource(self)
        self.with_raw_response = DodoPaymentsWithRawResponse(self)
        self.with_streaming_response = DodoPaymentsWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        environment: Literal["live_mode", "test_mode"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncDodoPayments(AsyncAPIClient):
    payments: payments.AsyncPaymentsResource
    subscriptions: subscriptions.AsyncSubscriptionsResource
    invoices: invoices.AsyncInvoicesResource
    licenses: licenses.AsyncLicensesResource
    license_keys: license_keys.AsyncLicenseKeysResource
    license_key_instances: license_key_instances.AsyncLicenseKeyInstancesResource
    customers: customers.AsyncCustomersResource
    refunds: refunds.AsyncRefundsResource
    disputes: disputes.AsyncDisputesResource
    payouts: payouts.AsyncPayoutsResource
    webhook_events: webhook_events.AsyncWebhookEventsResource
    products: products.AsyncProductsResource
    misc: misc.AsyncMiscResource
    discounts: discounts.AsyncDiscountsResource
    addons: addons.AsyncAddonsResource
    brands: brands.AsyncBrandsResource
    with_raw_response: AsyncDodoPaymentsWithRawResponse
    with_streaming_response: AsyncDodoPaymentsWithStreamedResponse

    # client options
    bearer_token: str

    _environment: Literal["live_mode", "test_mode"] | NotGiven

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        environment: Literal["live_mode", "test_mode"] | NotGiven = NOT_GIVEN,
        base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncDodoPayments client instance.

        This automatically infers the `bearer_token` argument from the `DODO_PAYMENTS_API_KEY` environment variable if it is not provided.
        """
        if bearer_token is None:
            bearer_token = os.environ.get("DODO_PAYMENTS_API_KEY")
        if bearer_token is None:
            raise DodoPaymentsError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the DODO_PAYMENTS_API_KEY environment variable"
            )
        self.bearer_token = bearer_token

        self._environment = environment

        base_url_env = os.environ.get("DODO_PAYMENTS_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `DODO_PAYMENTS_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "live_mode"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.payments = payments.AsyncPaymentsResource(self)
        self.subscriptions = subscriptions.AsyncSubscriptionsResource(self)
        self.invoices = invoices.AsyncInvoicesResource(self)
        self.licenses = licenses.AsyncLicensesResource(self)
        self.license_keys = license_keys.AsyncLicenseKeysResource(self)
        self.license_key_instances = license_key_instances.AsyncLicenseKeyInstancesResource(self)
        self.customers = customers.AsyncCustomersResource(self)
        self.refunds = refunds.AsyncRefundsResource(self)
        self.disputes = disputes.AsyncDisputesResource(self)
        self.payouts = payouts.AsyncPayoutsResource(self)
        self.webhook_events = webhook_events.AsyncWebhookEventsResource(self)
        self.products = products.AsyncProductsResource(self)
        self.misc = misc.AsyncMiscResource(self)
        self.discounts = discounts.AsyncDiscountsResource(self)
        self.addons = addons.AsyncAddonsResource(self)
        self.brands = brands.AsyncBrandsResource(self)
        self.with_raw_response = AsyncDodoPaymentsWithRawResponse(self)
        self.with_streaming_response = AsyncDodoPaymentsWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        environment: Literal["live_mode", "test_mode"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class DodoPaymentsWithRawResponse:
    def __init__(self, client: DodoPayments) -> None:
        self.payments = payments.PaymentsResourceWithRawResponse(client.payments)
        self.subscriptions = subscriptions.SubscriptionsResourceWithRawResponse(client.subscriptions)
        self.invoices = invoices.InvoicesResourceWithRawResponse(client.invoices)
        self.licenses = licenses.LicensesResourceWithRawResponse(client.licenses)
        self.license_keys = license_keys.LicenseKeysResourceWithRawResponse(client.license_keys)
        self.license_key_instances = license_key_instances.LicenseKeyInstancesResourceWithRawResponse(
            client.license_key_instances
        )
        self.customers = customers.CustomersResourceWithRawResponse(client.customers)
        self.refunds = refunds.RefundsResourceWithRawResponse(client.refunds)
        self.disputes = disputes.DisputesResourceWithRawResponse(client.disputes)
        self.payouts = payouts.PayoutsResourceWithRawResponse(client.payouts)
        self.webhook_events = webhook_events.WebhookEventsResourceWithRawResponse(client.webhook_events)
        self.products = products.ProductsResourceWithRawResponse(client.products)
        self.misc = misc.MiscResourceWithRawResponse(client.misc)
        self.discounts = discounts.DiscountsResourceWithRawResponse(client.discounts)
        self.addons = addons.AddonsResourceWithRawResponse(client.addons)
        self.brands = brands.BrandsResourceWithRawResponse(client.brands)


class AsyncDodoPaymentsWithRawResponse:
    def __init__(self, client: AsyncDodoPayments) -> None:
        self.payments = payments.AsyncPaymentsResourceWithRawResponse(client.payments)
        self.subscriptions = subscriptions.AsyncSubscriptionsResourceWithRawResponse(client.subscriptions)
        self.invoices = invoices.AsyncInvoicesResourceWithRawResponse(client.invoices)
        self.licenses = licenses.AsyncLicensesResourceWithRawResponse(client.licenses)
        self.license_keys = license_keys.AsyncLicenseKeysResourceWithRawResponse(client.license_keys)
        self.license_key_instances = license_key_instances.AsyncLicenseKeyInstancesResourceWithRawResponse(
            client.license_key_instances
        )
        self.customers = customers.AsyncCustomersResourceWithRawResponse(client.customers)
        self.refunds = refunds.AsyncRefundsResourceWithRawResponse(client.refunds)
        self.disputes = disputes.AsyncDisputesResourceWithRawResponse(client.disputes)
        self.payouts = payouts.AsyncPayoutsResourceWithRawResponse(client.payouts)
        self.webhook_events = webhook_events.AsyncWebhookEventsResourceWithRawResponse(client.webhook_events)
        self.products = products.AsyncProductsResourceWithRawResponse(client.products)
        self.misc = misc.AsyncMiscResourceWithRawResponse(client.misc)
        self.discounts = discounts.AsyncDiscountsResourceWithRawResponse(client.discounts)
        self.addons = addons.AsyncAddonsResourceWithRawResponse(client.addons)
        self.brands = brands.AsyncBrandsResourceWithRawResponse(client.brands)


class DodoPaymentsWithStreamedResponse:
    def __init__(self, client: DodoPayments) -> None:
        self.payments = payments.PaymentsResourceWithStreamingResponse(client.payments)
        self.subscriptions = subscriptions.SubscriptionsResourceWithStreamingResponse(client.subscriptions)
        self.invoices = invoices.InvoicesResourceWithStreamingResponse(client.invoices)
        self.licenses = licenses.LicensesResourceWithStreamingResponse(client.licenses)
        self.license_keys = license_keys.LicenseKeysResourceWithStreamingResponse(client.license_keys)
        self.license_key_instances = license_key_instances.LicenseKeyInstancesResourceWithStreamingResponse(
            client.license_key_instances
        )
        self.customers = customers.CustomersResourceWithStreamingResponse(client.customers)
        self.refunds = refunds.RefundsResourceWithStreamingResponse(client.refunds)
        self.disputes = disputes.DisputesResourceWithStreamingResponse(client.disputes)
        self.payouts = payouts.PayoutsResourceWithStreamingResponse(client.payouts)
        self.webhook_events = webhook_events.WebhookEventsResourceWithStreamingResponse(client.webhook_events)
        self.products = products.ProductsResourceWithStreamingResponse(client.products)
        self.misc = misc.MiscResourceWithStreamingResponse(client.misc)
        self.discounts = discounts.DiscountsResourceWithStreamingResponse(client.discounts)
        self.addons = addons.AddonsResourceWithStreamingResponse(client.addons)
        self.brands = brands.BrandsResourceWithStreamingResponse(client.brands)


class AsyncDodoPaymentsWithStreamedResponse:
    def __init__(self, client: AsyncDodoPayments) -> None:
        self.payments = payments.AsyncPaymentsResourceWithStreamingResponse(client.payments)
        self.subscriptions = subscriptions.AsyncSubscriptionsResourceWithStreamingResponse(client.subscriptions)
        self.invoices = invoices.AsyncInvoicesResourceWithStreamingResponse(client.invoices)
        self.licenses = licenses.AsyncLicensesResourceWithStreamingResponse(client.licenses)
        self.license_keys = license_keys.AsyncLicenseKeysResourceWithStreamingResponse(client.license_keys)
        self.license_key_instances = license_key_instances.AsyncLicenseKeyInstancesResourceWithStreamingResponse(
            client.license_key_instances
        )
        self.customers = customers.AsyncCustomersResourceWithStreamingResponse(client.customers)
        self.refunds = refunds.AsyncRefundsResourceWithStreamingResponse(client.refunds)
        self.disputes = disputes.AsyncDisputesResourceWithStreamingResponse(client.disputes)
        self.payouts = payouts.AsyncPayoutsResourceWithStreamingResponse(client.payouts)
        self.webhook_events = webhook_events.AsyncWebhookEventsResourceWithStreamingResponse(client.webhook_events)
        self.products = products.AsyncProductsResourceWithStreamingResponse(client.products)
        self.misc = misc.AsyncMiscResourceWithStreamingResponse(client.misc)
        self.discounts = discounts.AsyncDiscountsResourceWithStreamingResponse(client.discounts)
        self.addons = addons.AsyncAddonsResourceWithStreamingResponse(client.addons)
        self.brands = brands.AsyncBrandsResourceWithStreamingResponse(client.brands)


Client = DodoPayments

AsyncClient = AsyncDodoPayments



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel
from .currency import Currency
from .time_interval import TimeInterval
from .billing_address import BillingAddress
from .subscription_status import SubscriptionStatus
from .customer_limited_details import CustomerLimitedDetails

__all__ = ["SubscriptionListResponse"]


class SubscriptionListResponse(BaseModel):
    billing: BillingAddress

    created_at: datetime
    """Timestamp when the subscription was created"""

    currency: Currency

    customer: CustomerLimitedDetails

    metadata: Dict[str, str]

    next_billing_date: datetime
    """Timestamp of the next scheduled billing.

    Indicates the end of current billing period
    """

    on_demand: bool
    """Wether the subscription is on-demand or not"""

    payment_frequency_count: int
    """Number of payment frequency intervals"""

    payment_frequency_interval: TimeInterval

    previous_billing_date: datetime
    """Timestamp of the last payment. Indicates the start of current billing period"""

    product_id: str
    """Identifier of the product associated with this subscription"""

    quantity: int
    """Number of units/items included in the subscription"""

    recurring_pre_tax_amount: int
    """
    Amount charged before tax for each recurring payment in smallest currency unit
    (e.g. cents)
    """

    status: SubscriptionStatus

    subscription_id: str
    """Unique identifier for the subscription"""

    subscription_period_count: int
    """Number of subscription period intervals"""

    subscription_period_interval: TimeInterval

    tax_inclusive: bool
    """Indicates if the recurring_pre_tax_amount is tax inclusive"""

    trial_period_days: int
    """Number of days in the trial period (0 if no trial)"""

    cancelled_at: Optional[datetime] = None
    """Cancelled timestamp if the subscription is cancelled"""

    discount_id: Optional[str] = None
    """The discount id if discount is applied"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .license_key_status import LicenseKeyStatus

__all__ = ["LicenseKeyListParams"]


class LicenseKeyListParams(TypedDict, total=False):
    customer_id: Optional[str]
    """Filter by customer ID"""

    page_number: Optional[int]
    """Page number default is 0"""

    page_size: Optional[int]
    """Page size default is 10 max is 100"""

    product_id: Optional[str]
    """Filter by product ID"""

    status: Optional[LicenseKeyStatus]
    """Filter by license key status"""
